#!/usr/bin/php
<?php

#
# cacti script to read ucd/net-snmp load average data with SNMP
#
include(dirname(__FILE__) . "/../include/global.php");
include(dirname(__FILE__) . "/../lib/snmp.php");

#
# global declarations
#
global $snmp_hostname;
global $snmp_community;
global $snmp_version;
global $snmp_username;
global $snmp_password;
global $snmp_port;
global $snmp_timeout;
global $snmp_array;
global $oid;
global $cacti_request;
global $cacti_request_type;
global $cacti_request_index;
global $cmdtool;
global $cmdoutput;
global $cmdoutputcount;
global $output;
global $outputcount;
global $cpu_array;
global $cpu_index;
global $cpu_name;
global $cpu_desc;
global $cpu_1min;
global $cpu_5min;

#
# make sure enough parameters were provided
#
if (count($_SERVER['argv']) < 8) {

	echo ("failed: insufficient parameters provided\n");

	displaySyntax();

	exit(1);
}

#
# 1st parameter is always $snmp_hostname
#
$snmp_hostname = substr($_SERVER['argv'][1],0,(strlen($_SERVER['argv'][1]) - 1));

if ($snmp_hostname == "") {

	echo ("failed: invalid hostname\n");

	displaySyntax();

	exit(1);
}

#
# 2nd parameter is always $snmp_community; testing is done after we have the version
#
$snmp_community = substr($_SERVER['argv'][2],0,(strlen($_SERVER['argv'][2]) - 1));

#
# 3rd parameter is always $snmp_version
#
$snmp_version = substr($_SERVER['argv'][3],0,(strlen($_SERVER['argv'][3]) - 1));

if (($snmp_version != 1) and ($snmp_version != 2) and ($snmp_version != 3)) {

	echo ("failed: \"$snmp_version\" is not a valid SNMP version\n");

	displaySyntax();

	exit(1);
}

if (($snmp_version != 3) and ($snmp_community == "")) {

	echo ("failed: invalid SNMP community string\n");

	displaySyntax();

	exit(1);
}

#
# 4th parameter is always $snmp_username
#
$snmp_username = substr($_SERVER['argv'][4],0,(strlen($_SERVER['argv'][4]) - 1));

#
# 5th parameter is always $snmp_password
#
$snmp_password = substr($_SERVER['argv'][5],0,(strlen($_SERVER['argv'][5]) - 1));

#
# 6th parameter is always $snmp_port
#
$snmp_port = substr($_SERVER['argv'][6],0,(strlen($_SERVER['argv'][6]) - 1));

if (is_numeric($snmp_port) == FALSE) {

	echo ("failed: \"$snmp_port\" is not a numeric port number \n");

	displaySyntax();

	exit(1);
}

#
# 7th parameter is always $snmp_timeout
#
$snmp_timeout = substr($_SERVER['argv'][7],0,(strlen($_SERVER['argv'][7]) - 1));

if (is_numeric($snmp_timeout) == FALSE) {

	echo ("failed: \"$snmp_timeout\" is not a numeric timeout value \n");

	displaySyntax();

	exit(1);
}

#
# Cacti's SNMP timeout uses milliseconds, while PHP uses the value
# from Net-SNMP's header files, which is typically microseconds.
# Normalize by multiplying the timeout parameter value by 1000.
#
$snmp_timeout = ($snmp_timeout * 1000);

#
# 8th parameter is always $cacti_request
#
$cacti_request = strtolower($_SERVER['argv'][8]);

if (($cacti_request != "index") &&
	($cacti_request != "query") &&
	($cacti_request != "get")) {

	echo ("failed: unknown cacti request\n");

	displaySyntax();

	exit(1);
}

#
# read the rest of the cacti request
#
if (($cacti_request == "query") ||
	($cacti_request == "get")) {

	if (isset($_SERVER['argv'][9]) === FALSE) {

		echo ("failed: incomplete cacti request\n");

		displaySyntax();

		exit(1);
	}

	else {
		$cacti_request_type = strtolower($_SERVER['argv'][9]);

		if (($cacti_request_type != "cpuindex") &&
			($cacti_request_type != "cpuname") &&
			($cacti_request_type != "cpudesc") &&
			($cacti_request_type != "onemin") &&
			($cacti_request_type != "fivemin")) {

			echo ("failed: unknown cacti request type\n");

			displaySyntax();

			exit(1);
		}
	}
}

if ($cacti_request == "get") {

	if (isset($_SERVER['argv'][10]) === FALSE) {

		echo ("failed: incomplete cacti request\n");

		displaySyntax();

		exit(1);
	}

	else {

		$cacti_request_index = $_SERVER['argv'][10];

		if (($cacti_request_index == "") ||
			($cacti_request_index < 1)) {

			echo ("failed: unknown cacti index value\n");
		}
	}
}

#
# define a nested array of the OIDs and their labels
#
$oid = array (
	array ("cpuIndex", ".1.3.6.1.4.1.9.9.109.1.1.1.1.2"),
	array ("cpuDescr", ".1.3.6.1.2.1.47.1.1.1.1.7"),
	array ("cpu_1min", ".1.3.6.1.4.1.9.9.109.1.1.1.1.7"),
	array ("cpu_5min", ".1.3.6.1.4.1.9.9.109.1.1.1.1.8"));

$cmdoutputcount = 0;

#
# walk the index OID to create a scratch array of known entries
#
$snmp_array = cacti_snmp_walk($snmp_hostname,
	$snmp_community,
	$oid[0][1],
	$snmp_version,
	$snmp_username,
	$snmp_password, "", "", "", "",
	$snmp_port,
	$snmp_timeout);

#
# fill in the cmdoutput array with details for each discovered CPU
#
while ($cmdoutputcount < count($snmp_array)) {

	#
	# grab the index OID, which will be used to close all other requests
	#
	if (preg_match('/(\d+)$/', $snmp_array[$cmdoutputcount]['oid'], $scratch) == 1) {

		$cmdoutput[$cmdoutputcount][0] = $scratch[1];
	}

	else {
		echo ("failed: unable to determine CPU index \n");

		exit(1);
	}

	#
	# grab the CPU type value from the index OID
	#
	if ((isset($snmp_array[$cmdoutputcount]['value']) === TRUE) &&
		(is_numeric($snmp_array[$cmdoutputcount]['value']) === TRUE)) {

		$cmdoutput[$cmdoutputcount][1] = $snmp_array[$cmdoutputcount]['value'];
	}

	else {
		$cmdoutput[$cmdoutputcount][1] == 0;
	}

	#
	# find the CPU description associated with the type
	#
	# if the CPU type is unknown, call it unknown
	#
	if ($cmdoutput[$cmdoutputcount][1] == 0) {

		$cmdoutput[$cmdoutputcount][2] = "onboard CPU";
	}

	else {
		
		$cmdoutput[$cmdoutputcount][2] = cacti_snmp_get($snmp_hostname,
			$snmp_community,
			$oid[1][1] . $cmdoutput[$cmdoutputcount][1],
			$snmp_version,
			$snmp_username,
			$snmp_password, "", "", "", "",
			$snmp_port,
			$snmp_timeout);
	}

	#
	# fetch the 1min and 5min values
	#
	$cmdoutput[$cmdoutputcount][3] = cacti_snmp_get($snmp_hostname,
		$snmp_community,
		($oid[2][1] . "." . $cmdoutput[$cmdoutputcount][0]),
		$snmp_version,
		$snmp_username,
		$snmp_password, "", "", "", "",
		$snmp_port,
		$snmp_timeout);

	#
	# if the 1min OID failed, use the old OID
	#
	if ((isset($cmdoutput[$cmdoutputcount][3]) === FALSE) ||
		(is_numeric($cmdoutput[$cmdoutputcount][3]) === FALSE)) {

		$oid[2][1] = ".1.3.6.1.4.1.9.9.109.1.1.1.1.4";
		$oid[3][1] = ".1.3.6.1.4.1.9.9.109.1.1.1.1.4";

		$cmdoutput[$cmdoutputcount][3] = cacti_snmp_get($snmp_hostname,
			$snmp_community,
			($oid[2][1] . "." . $cmdoutput[$cmdoutputcount][0]),
			$snmp_version,
			$snmp_username,
			$snmp_password, "", "", "", "",
			$snmp_port,
			$snmp_timeout);
	}

	$cmdoutput[$cmdoutputcount][4] = cacti_snmp_get($snmp_hostname,
		$snmp_community,
		($oid[3][1] . "." . $cmdoutput[$cmdoutputcount][0]),
		$snmp_version,
		$snmp_username,
		$snmp_password, "", "", "", "",
		$snmp_port,
		$snmp_timeout);
	
	#
	# do the next output entry
	#
	$cmdoutputcount++;
}

#
# make a clean array
#
$cpu_array = $cmdoutput;

#
# generate output
#
$outputcount = 0;

while ($outputcount < count($cpu_array)) {

	$cpu_index = $cpu_array[$outputcount][0];
	$cpu_name = "CPU" . $cpu_array[$outputcount][0];
	$cpu_desc = $cpu_array[$outputcount][2];
	$cpu_1min = $cpu_array[$outputcount][3];
	$cpu_5min = $cpu_array[$outputcount][4];

	#
	# if they want an index, dump a list of numbers
	#
	if ($cacti_request == "index") {

		echo ($cpu_index . "\n");

		$outputcount++;
	}

	#
	# if they want query data, give them the type they want
	#
	elseif ($cacti_request == "query") {

		#
		# they want the index numbers
		#
		if ($cacti_request_type == "cpuindex") {

			echo ($cpu_index . ":" . $cpu_index . "\n");

			$outputcount++;
		}

		#
		# they want the cpu names
		#
		elseif ($cacti_request_type == "cpuname") {

			echo ($cpu_index . ":" . $cpu_name . "\n");

			$outputcount++;
		}

		#
		# they want the cpu description
		#
		elseif ($cacti_request_type == "cpudesc") {

			echo ($cpu_index . ":" . $cpu_desc . "\n");

			$outputcount++;
		}

		#
		# they want the 1 min averages
		#
		elseif ($cacti_request_type == "onemin") {

			echo ($cpu_index . ":" . $cpu_1min . "\n");

			$outputcount++;
		}

		#
		# they want the 5 min averages
		#
		elseif ($cacti_request_type == "fivemin") {

			echo ($cpu_index . ":" . $cpu_5min . "\n");

			$outputcount++;
		}

		else {
			displaySyntax();

			exit(1);
		}
	}

	#
	# if they want a specific entry, find it and give them the type they want
	#
	elseif ($cacti_request == "get") {

		#
		# they want the index number back
		#
		if (($cacti_request_type == "cpuindex") &&
			($cacti_request_index == $cpu_index)) {

			echo ($cpu_index . "\n");

			break;
		}

		#
		# they want the cpu name
		#
		elseif (($cacti_request_type  == "cpuname") &&
			($cacti_request_index  == $cpu_index)) {

			echo ($cpu_name . "\n");

			break;
		}

		#
		# they want the cpu description
		#
		elseif (($cacti_request_type  == "cpudesc") &&
			($cacti_request_index  == $cpu_index)) {

			echo ($cpu_desc . "\n");

			break;
		}

		#
		# they want the 1 min average
		#
		elseif (($cacti_request_type  == "onemin") &&
			($cacti_request_index  == $cpu_index)) {

			echo ($cpu_1min . "\n");

			break;
		}

		#
		# they want the 5 min average
		#
		elseif (($cacti_request_type  == "fivemin") &&
			($cacti_request_index  == $cpu_index)) {

			echo ($cpu_5min . "\n");

			break;
		}

		else {
			$outputcount++;
		}
	}

	else {
		echo ("failed: unknown cacti request\n");

		displaySyntax();

		exit(1);
	}
}

exit(0);

function displaySyntax() {

	#
	# display the syntax
	#
	echo ("syntax: " . $_SERVER['argv'][0] . " <hostname>, [<snmp_community>], <snmp_version>,\n" .
		"    [<snmp3_username>], [<snmp3_password>], [<snmp_port>], <snmp_timeout>,\n" .
		"    (index|query <fieldname>|get <fieldname> <index>)\n");
}

?>
